/*
 * $Id: Background.as,v 1.1.1.1 2007/04/01 08:25:41 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.la2 {
  import flash.display.BitmapData;
  import flash.geom.Rectangle;
  import abagames.util.Vector;

  /**
   * Buildings and shadows on a background.
   */
  public class Background {
    public static const HEIGHT_MAP_SQUARE_SIZE:Number = 20;
    private var buffer:BitmapData;
    private var field:Field;
    private var heightMap:Array;
    private var heightMapX:Number;
    private var heightMapY:Number;
    private var heightMapYScroll:Number;
    private var heightMapWidth:int;
    private var heightMapHeight:int;
    private var heightMapSquareRect:Rectangle;
    private var shadows:Array;
    private var shadowIdx:int;
    private var shadowIdxNext:int;
    private var shadowRect:Rectangle;

    public function Background(buffer:BitmapData, field:Field) {
      this.buffer = buffer;
      this.field = field;
      heightMapWidth = field.size.x / HEIGHT_MAP_SQUARE_SIZE;
      heightMapHeight = field.size.y / HEIGHT_MAP_SQUARE_SIZE;
      heightMap = new Array;
      for (var hx:int = 0; hx <= heightMapWidth; hx++) {
        var hl:Array = new Array;
        for (var hy:int = 0; hy <= heightMapHeight; hy++) {
          var h:Number = 0;
          if (hx % 10 != 0 && hy % 10 != 0)
            h = Math.random() * 25.0;
          hl.push(h);
        }
        heightMap.push(hl);
      }
      heightMapSquareRect = new Rectangle(0, 0, HEIGHT_MAP_SQUARE_SIZE, HEIGHT_MAP_SQUARE_SIZE);
      shadows = new Array;
      for (var i:int = 0; i < 1024; i++)
        shadows.push(new Shadow());
      shadowIdx = shadowIdxNext = 0;
      shadowRect = new Rectangle;
      heightMapX = heightMapY = heightMapYScroll = 0;
    }

    public function update():void {
      heightMapX = field.offset.x * 0.5;
      heightMapY = field.offset.y * 0.5;
      var htmy:Number = heightMapY + heightMapYScroll;
      if (htmy < 0)
        htmy += field.size.y;
      if (htmy >= field.size.y)
        htmy -= field.size.y;
      var hmx:int;
      var shmx:int = heightMapX / HEIGHT_MAP_SQUARE_SIZE;
      var hmxo:Number = heightMapX % HEIGHT_MAP_SQUARE_SIZE;
      var hmy:int = htmy / HEIGHT_MAP_SQUARE_SIZE;
      var hmyo:Number = htmy % HEIGHT_MAP_SQUARE_SIZE;
      for (var hy:int = 0; hy <= heightMapHeight / Field.SIZE_RATIO + 1; hy++) {
        hmx = shmx;
        for (var hx:int = 0; hx <= heightMapWidth / Field.SIZE_RATIO + 1; hx++) {
          var h:Number = heightMap[hmx][hmy];
          heightMapSquareRect.x = hx * HEIGHT_MAP_SQUARE_SIZE - hmxo - h;
          heightMapSquareRect.y = hy * HEIGHT_MAP_SQUARE_SIZE - hmyo - h;
          var ox:Number = hx * HEIGHT_MAP_SQUARE_SIZE - hmxo - 200;
          var oy:Number = hy * HEIGHT_MAP_SQUARE_SIZE - hmyo - 150;
          var a:Number = Math.sqrt(ox * ox + oy * oy);
          a *= 0.012;
          a = 1.5 - a;
          if (a > 1)
            a = 1;
          if (a > 0) {
            var c:int = int(0x55 * a) * 0x10000 + int(0x33 * a) * 0x100 + int(0x99 * a);
            buffer.fillRect(heightMapSquareRect, c);
          }
          hmx++;
        }
        hmy++;
        if (hmy >= heightMapHeight)
          hmy -= heightMapHeight;
      }
      drawShadows();
      heightMapYScroll -= 0.1;
      if (heightMapYScroll < 0)
        heightMapYScroll += field.size.y;
    }

    public function addShadow(x:Number, y:Number, size:Vector):void {
      shadows[shadowIdxNext].set(x, y, size.x, size.y);
      shadowIdxNext--;
      if (shadowIdxNext < 0)
        shadowIdxNext += shadows.length;
    }

    public function drawShadows():void {
      for (var i:int = 0; i < shadows.length; i++) {
        if (shadowIdx == shadowIdxNext)
          break;
        var s:Shadow = shadows[shadowIdx];
        var sx:Number = s.pos.x * 0.5 - heightMapX;
        var hmx:int = s.pos.x * 0.5 / HEIGHT_MAP_SQUARE_SIZE;
        var sy:Number = s.pos.y * 0.5 - heightMapY;
        var hy:Number = s.pos.y * 0.5 - heightMapY;
        if (hy < 0)
          hy += field.size.y;
        var hmy:int = hy / HEIGHT_MAP_SQUARE_SIZE;
        var h:Number = heightMap[hmx][hmy];
        shadowRect.x = sx - h;
        shadowRect.y = sy - h;
        shadowRect.width = s.size.x * 0.5;
        shadowRect.height = s.size.y * 0.5;
        buffer.fillRect(shadowRect, 0);
        shadowIdx--;
        if (shadowIdx < 0)
          shadowIdx += shadows.length;
      }
    }
  }
}

import abagames.util.Vector;

class Shadow {
  public var pos:Vector;
  public var size:Vector;

  public function Shadow() {
    pos = new Vector;
    size = new Vector;
  }

  public function set(x:Number, y:Number, w:Number, h:Number):void {
    pos.x = x + 250;
    pos.y = y + 150;
    size.x = w;
    size.y = h;
  }
}
