/*
 * $Id: MainLoop.as,v 1.2 2007/04/04 13:08:57 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.la2 {
  import flash.display.Sprite;
  import flash.display.Bitmap;
  import flash.display.BitmapData;
  import flash.geom.Rectangle;
  import flash.events.Event;
  import flash.media.Sound;
  import flash.media.SoundChannel;
  import flash.media.SoundTransform;
  import abagames.util.KeyState;

  /**
   * Main loop that updates a game's state and handles actor pools.
   * Bgm and SEs are also handled in this class.
   */
  public class MainLoop {
    private var rootSprite:Sprite;
    private var buffer:BitmapData;
    private var screen:Bitmap;
    private var keyState:KeyState;
    private var field:Field;
    private var background:Background;
    private var player:Player;
    private var turrets:TurretPool;
    private var bullets:BulletPool;
    private var particles:ParticlePool;
    private var scoreboards:ScoreboardPool;
    private var sounds:Array;
    private var bgmChannel:SoundChannel;
    private var bgmTransform:SoundTransform;
    private var bgmFadeout:Boolean;

    public function init(rootSprite:Sprite, keyState:KeyState, buffer:BitmapData, screen:Bitmap, sounds:Array):void {
      this.rootSprite = rootSprite;
      this.keyState = keyState;
      this.buffer = buffer;
      this.screen = screen;
      this.sounds = sounds;
      field = new Field(buffer);
      background = new Background(buffer, field);
      particles = new ParticlePool(512, buffer, field, background);
      scoreboards = new ScoreboardPool(256, field);
      player = new Player(buffer, keyState, field, background, this);
      bullets = new BulletPool(256, buffer, player, field, background, particles);
      turrets = new TurretPool(8, buffer, player, field, bullets, particles);
      player.setParams(bullets, particles, turrets);
      field.setParams(turrets, bullets, particles, scoreboards, background, player, keyState, this);
      Letter.init(buffer, field, background);
      startTitle();
      bgmTransform = new SoundTransform;
    }

    public function startTitle():void {
      initPools();
      player.init();
      player.setHidden();
      field.startTitle();
      stopBgm();
    }

    public function startInGame():void {
      initPools();
      player.init();
      field.startInGame();
      startBgm();
    }

    private function initPools():void {
      particles.clear();
      scoreboards.clear();
      bullets.clear();
      turrets.clear();
    }

    private function startBgm():void {
      bgmTransform.volume = 1;
      bgmFadeout = false;
      bgmChannel = sounds[0].play(0, 99999, bgmTransform);
    }

    public function fadeoutBgm():void {
      bgmFadeout = true;
    }

    public function stopBgm():void {
      bgmFadeout = false;
      if (bgmChannel != null)
        bgmChannel.stop();
      bgmChannel = null;
    }

    public function playSe(n:int, loop:int = 1, st:SoundTransform = null):SoundChannel {
      return sounds[n].play(0, loop, st);
    }

    public function update():void {
      buffer.fillRect(buffer.rect, 0);
      if (bgmChannel != null) {
        Shape.distortion = 0.7 + (bgmChannel.leftPeak + bgmChannel.rightPeak) * 0.3;
        if (bgmFadeout) {
          bgmTransform.volume -= 0.01;
          if (bgmTransform.volume <= 0.05) {
            bgmTransform.volume = 0;
            bgmFadeout = false;
          }
          bgmChannel.soundTransform = bgmTransform;
        }
      }
      background.update();
      particles.update();
      scoreboards.update();
      field.update();
      turrets.update();
      bullets.update();
      player.update();
      field.drawStatus();
    }
  }
}
