/*
 * $Id: Player.as,v 1.2 2007/04/04 13:08:57 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.la2 {
  import flash.display.Sprite;
  import flash.display.Bitmap;
  import flash.display.BitmapData;
  import flash.geom.Matrix;
  import flash.media.SoundChannel;
  import flash.media.SoundTransform;
  import abagames.util.Vector;
  import abagames.util.KeyState;

  /**
   * Player's ship that fires a glider.
   */
  public class Player {
    public static const BULLET_HIT_SIZE:Number = 3.0;
    private static const DEFAULT_SPEED:Number = 4;
    private static const DEG_VELOCITY:Number = 0.2;
    private static const RESTART_CNT:int = 268;
    private static const INVINCIBLE_CNT:int = 228;
    private static const FIRE_INTERVAL:int = 8;
    private var buffer:BitmapData;
    private var field:Field;
    private var mainLoop:MainLoop;
    private var bullets:BulletPool;
    private var particles:ParticlePool;
    private var turrets:TurretPool;
    private var _pos:Vector;
    private var deg:Number;
    private var keyState:KeyState;
    private var invincibleCnt:int;
    private var shape:Shape;
    private var aPressed:Boolean;
    private var speed:Number;
    private var flyingSe:SoundChannel;
    private var flyingSeTransform:SoundTransform;

    public function Player(buffer:BitmapData, keyState:KeyState, field:Field, background:Background, mainLoop:MainLoop) {
      this.buffer = buffer;
      this.keyState = keyState;
      this.field = field;
      this.mainLoop = mainLoop;
      _pos = new Vector;
      shape = new Shape(buffer, field, background);
      shape.addBox(0, 0, 10, 10, 2, 255, 0, 0);
      shape.addBox(-4, -8, 6, 6, 1, 255, 150, 0);
      shape.addBox(4, -8, 6, 6, 1, 255, 150, 0);
      flyingSeTransform = new SoundTransform;
    }

    public function setParams(bullets:BulletPool, particles:ParticlePool, turrets:TurretPool):void {
      this.bullets = bullets,
      this.particles = particles;
      this.turrets = turrets;
    }

    public function init():void {
      respawn();
    }

    public function respawn():void {
      _pos.x = field.size.x * 0.5;
      _pos.y = field.size.y * 0.5;
      setFieldOffset();
      deg = 0;
      invincibleCnt = INVINCIBLE_CNT;
      aPressed = true;
      speed = DEFAULT_SPEED;
      stopFlyingSe();
    }

    public function setHidden():void {
      invincibleCnt = 999999;
    }

    public function update():void {
      if (invincibleCnt > INVINCIBLE_CNT) {
        if (invincibleCnt < 10000) {
          invincibleCnt--;
          if (invincibleCnt == INVINCIBLE_CNT)
            respawn();
        }
        return;
      }
      var vx:Number = 0;
      var vy:Number = 0;
      if (keyState.left)
        vx = -speed;
      if (keyState.right)
        vx = speed;
      if (keyState.up)
        vy = -speed;
      if (keyState.down)
        vy = speed;
      if (vx != 0 && vy != 0) {
        vx *= 0.7;
        vy *= 0.7;
      }
      _pos.x += vx;
      _pos.y += vy;
      if (_pos.x < 8)
        _pos.x = 8;
      if (_pos.x > field.size.x - 16)
        _pos.x = field.size.x - 16;
      if (_pos.y < 8)
        _pos.y = 8;
      if (_pos.y > field.size.y - 16)
        _pos.y = field.size.y - 16;
      setFieldOffset();
      if (vx != 0 || vy != 0) {
        var td:Number = normalizeDeg(Math.atan2(vx, vy));
        var od:Number = normalizeDeg(deg - td);
        if (od > DEG_VELOCITY)
          deg -= DEG_VELOCITY;
        else if (od < -DEG_VELOCITY)
          deg += DEG_VELOCITY;
        else
          deg = td;
        deg = normalizeDeg(deg);
        var p:Particle = Particle(particles.getInstanceForced());
        p.add(_pos.x, _pos.y, deg + Math.PI + Math.random() * 0.6 - 0.3, speed * Math.random(),
              255, 100, 0);
      }
      if (keyState.a) {
        if (!aPressed) {
          aPressed = true;
          field.addGlider(_pos, deg);
        } else {
          if (field.taintedCellsNum <= 0) {
            stopFlyingSe();
          } else {
            var vol:Number = 0.33 + field.taintedCellsNum * 0.02;
            if (vol > 1)
              vol = 1;
            flyingSeTransform.volume = vol;
            flyingSeTransform.pan =
              (field.taintedCellsCenterPosX - field.cellsWidth / 2) /
                (field.cellsWidth / 2 + 1);
            if (flyingSe == null)
              flyingSe = mainLoop.playSe(1, 9999, flyingSeTransform);
            if (flyingSe != null)
              flyingSe.soundTransform = flyingSeTransform;
          }
        }
        speed += (DEFAULT_SPEED * 0.66 - speed) * 0.1;
      } else {
        if (aPressed) {
          aPressed = false;
          if (field.removeTaintedCells() > 0)
            mainLoop.playSe(2, 1, flyingSeTransform);
          stopFlyingSe();
        }
        speed += (DEFAULT_SPEED - speed) * 0.1;
      }
      var visible:Boolean = true;
      if (invincibleCnt > 0) {
        invincibleCnt--;
        if (invincibleCnt % 30 >= 15)
          visible = false;
      } else {
        if (field.existsNotTaintedCellScreen(pos) || bullets.checkHitToPlayer(pos)) {
          destroyed();
          return;
        }
      }
      if (visible)
        shape.drawTilt(_pos, deg);
    }

    private function setFieldOffset():void {
      field.offset.x = field.size.x * Field.SIZE_OFFSET_RATIO / 2 + (_pos.x - field.size.x / 2) * 0.8;
      field.offset.y = field.size.y * Field.SIZE_OFFSET_RATIO / 2 + (_pos.y - field.size.y / 2) * 0.8;
      if (field.offset.x < 0)
        field.offset.x = 0;
      if (field.offset.x >= field.size.x * Field.SIZE_OFFSET_RATIO)
        field.offset.x = field.size.x * Field.SIZE_OFFSET_RATIO;
      if (field.offset.y < 0)
        field.offset.y = 0;
      if (field.offset.y >= field.size.y * Field.SIZE_OFFSET_RATIO)
        field.offset.y = field.size.y * Field.SIZE_OFFSET_RATIO;
    }

    public function drawShape(p:Vector):void {
      shape.draw(p, 1, false);
    }

    private function normalizeDeg(d:Number):Number {
      if (d > Math.PI)
        return d - Math.PI * 2;
      else if (d < -Math.PI)
        return d + Math.PI * 2;
      else
        return d;
    }

    private function destroyed():void {
      stopFlyingSe();
      mainLoop.playSe(3);
      turrets.removeAll();
      var hl:Boolean = field.destroyedPlayer();
      for (var i:int = 0; i < 128; i++) {
        var p:Particle = Particle(particles.getInstanceForced());
        p.add(_pos.x, _pos.y, Math.random() * Math.PI * 2, Math.random() * 10 + 1,
              Math.random() * 128 + 128, 200, 200);
      }
      if (hl) {
        field.resetDifficulty();
        invincibleCnt = RESTART_CNT;
      } else {
        setHidden();
      }
    }

    private function stopFlyingSe():void {
      if (flyingSe != null) {
        flyingSe.stop();
        flyingSe = null;
      }
    }

    public function get pos():Vector {
      return _pos;
    }
  }
}
